def satellite_initial_positions(inputs):
    '''
    Calculates the initial x,y,z of each satellite.
    If satellite motion is included, then it gives the locations of the
    satellites at each time during the simulation.

    Going to wait until I have some satellites to deal with to write this.
    '''

    time_given = 'time' in inputs.geometry

    return None
